"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const service_context_1 = require("./service/service-context");
const mobile_devices_controller_1 = require("mobile-devices-controller");
class DeviceManger {
    constructor(port, _serveiceContext = undefined) {
        this._serveiceContext = _serveiceContext;
        if (!this._serveiceContext) {
            this._serveiceContext = service_context_1.ServiceContext.createServer(port);
        }
    }
    startDevice(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let device = DeviceManger.getDefaultDevice(args);
            // When isSauceLab specified we simply do nothing;
            if (args.isSauceLab || args.ignoreDeviceController) {
                DeviceManger._emulators.set(args.runType, device);
                return device;
            }
            // Using serve to manage deivces.
            if (args.useDeviceControllerServer) {
                const d = yield this._serveiceContext.subscribe(args.appiumCaps.deviceName, args.appiumCaps.platformName.toLowerCase(), args.appiumCaps.platformVersion, args.appiumCaps.app);
                if (!d || !d) {
                    console.error("", d);
                    throw new Error("Missing device: " + d);
                }
                DeviceManger._emulators.set(args.runType, d);
                return d;
            }
            const allDevices = (yield mobile_devices_controller_1.DeviceController.getDevices({ platform: args.appiumCaps.platformName }));
            if (!allDevices || allDevices === null || allDevices.length === 0) {
                console.log("We couldn't find any devices. We will try to proceed to appium! Maybe avd manager is missing");
                console.log("Available devices:\n", allDevices);
            }
            const searchObj = args.appiumCaps.udid ? { token: args.appiumCaps.udid } : { name: args.appiumCaps.deviceName, apiLevel: args.appiumCaps.platformVersion };
            let searchedDevices = mobile_devices_controller_1.DeviceController.filter(allDevices, searchObj);
            if (!searchedDevices || searchedDevices.length === 0) {
                console.log(`No such device ${args.appiumCaps.deviceName}!!!\n Check your device name!!!`);
                console.log("Available devices:\n", allDevices);
            }
            if (searchedDevices && searchedDevices.length > 0) {
                // Should find new device
                if (!args.reuseDevice) {
                    device = mobile_devices_controller_1.DeviceController.filter(searchedDevices, { status: mobile_devices_controller_1.Status.SHUTDOWN })[0];
                }
                // If there is no shutdown device
                if (!device || device === null || !device.status) {
                    device = mobile_devices_controller_1.DeviceController.filter(searchedDevices, { status: mobile_devices_controller_1.Status.BOOTED })[0];
                }
                // In case reuse device is true but there weren't any booted devices. We need to fall back and boot new one.
                if (!device || device === null && args.reuseDevice) {
                    device = mobile_devices_controller_1.DeviceController.filter(searchedDevices, { status: mobile_devices_controller_1.Status.SHUTDOWN })[0];
                }
                if (device.status === mobile_devices_controller_1.Status.SHUTDOWN) {
                    yield mobile_devices_controller_1.DeviceController.startDevice(device);
                    console.log("Started device: ", device);
                }
                else {
                    console.log("Device is already started", device);
                    if (!args.reuseDevice && device.type !== mobile_devices_controller_1.DeviceType.EMULATOR && device.type !== mobile_devices_controller_1.DeviceType.SIMULATOR) {
                        console.log("Since is it specified without reusing, the device would be shut down and restart!");
                        mobile_devices_controller_1.DeviceController.kill(device);
                        yield mobile_devices_controller_1.DeviceController.startDevice(device);
                    }
                }
            }
            DeviceManger._emulators.set(args.runType, device);
            return device;
        });
    }
    stopDevice(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (args.useDeviceControllerServer) {
                const device = DeviceManger._emulators.get(args.runType);
                const d = yield this._serveiceContext.unsubscribe(device.token);
                if (!d) {
                    console.error("", d);
                    throw new Error("Missing device: " + d);
                }
                try {
                    yield this._serveiceContext.releasePort(args.port);
                    if (args.appiumCaps.wdaLocalPort) {
                        yield this._serveiceContext.releasePort(args.appiumCaps.wdaLocalPort);
                    }
                }
                catch (error) {
                    console.log(error);
                }
            }
            if (DeviceManger._emulators.has(args.runType) && !args.reuseDevice && !args.isSauceLab && !args.ignoreDeviceController && args.useDeviceControllerServer) {
                const device = DeviceManger._emulators.get(args.runType);
                yield DeviceManger.kill(device);
            }
        });
    }
    static kill(device) {
        return __awaiter(this, void 0, void 0, function* () {
            yield mobile_devices_controller_1.DeviceController.kill(device);
        });
    }
    static getDefaultDevice(args) {
        return new mobile_devices_controller_1.Device(args.appiumCaps.deviceName, args.appiumCaps.platformVersion, undefined, args.appiumCaps.platformName, undefined, undefined);
    }
}
DeviceManger._emulators = new Map();
exports.DeviceManger = DeviceManger;
//# sourceMappingURL=device-controller.js.map